# frozen_string_literal: true

#
# Cookbook:: snu_nginx_fips
# Library:: resource/snu_nginx_fips_service_upstart
#
# Copyright:: 2017, Socrata, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require 'chef/platform/service_helpers'
require 'chef/provider/service/upstart'
require 'chef/resource'
require 'chef/resource/service'

class Chef
  class Resource
    #
    # An implemetation of the snu_nginx_fips_service resource for platforms
    # with Upstart.
    #
    class SnuNginxFipsServiceUpstart < SnuNginxFipsService
      provides :snu_nginx_fips_service_upstart
      provides :snu_nginx_fips_service do |_node|
        provs = Chef::Platform::ServiceHelpers.service_resource_providers
        provs.include?(:upstart) && !provs.include?(:systemd)
      end

      #
      # The :create action needs to define an Upstart config file for Nginx.
      #
      action :create do
        link '/etc/init/nginx.conf' do
          only_if { ::File.symlink?('/etc/init/nginx.conf') }
          action :delete
        end

        file '/etc/init/nginx.conf' do
          owner 'root'
          group 'root'
          mode '0644'
          content <<-CONTENT.gsub(/^ {12}/, '')
            description "The Nginx web server"

            start on (filesystem and net-device-up IFACE!=lo)
            stop on runlevel [!2345]

            expect fork
            respawn
            respawn limit 10 5
            limit nofile #{new_resource.file_limit} #{new_resource.file_limit}

            exec /usr/sbin/nginx -c #{new_resource.config_file}
          CONTENT
        end
      end

      #
      # The :remove action needs to delete the Upstart config file for Nginx.
      #
      action :remove do
        file('/etc/init/nginx.conf') { action :delete }
      end

      #
      # RHEL6 has Upstart support but Chef won't use it unless we specify the
      # provider.
      #
      Chef::Resource::Service.allowed_actions.each do |a|
        action a do
          super()
          edit_resource!(:service, "#{a} nginx") do
            provider Chef::Provider::Service::Upstart
          end
        end
      end
    end
  end
end
